/* 1-July-2005 Drew Baden

This module duplicates some of what is in Tullio's TTCdecoder to be able
to see that the sandwich board is seeing the right TTC broadcasts

We _ignore_ the bits BRCST<2> and BRCST<4>.  These bits cause
specific behavior on the FrontEnd.  We may or may not want to associate
this behavior with any of these messages, so we should conside these
bits as "don't care".  We are then left with

765X3XXX

since the low two bits are also special -- EC reset and BC reset.

Here is the mapping:
                          765X3XXX
 BCZero                   000X1XXX
 Test Enable              001X0XXX
 ResetOrbitCounter        001X1XXX
 Private Gap              010X0XXX
 ReSync                   010X1XXX
 Private Orbit            011X0XXX
 HardReset                011X1XXX
 SoftReset                100X0XXX
 Start                    100X1XXX
 StRq                     101X0XXX
 Stop                     101X1XXX
 Asych Fifo Reset         110X0XXX
 Calib Trigger            110X1XXX

 This mapping leaves six codes available for any last-minute functionality.

NB: all TTC_input are synchronous with TTC_CK40Des1 and at some point
the signals must cross to the domain of the system clock (clk).
The best strategy seems to have only 1-bit 1-tick signals (not busses)
crossing the clock domains; it would be a problem if a bus
crosses a clock domain while not all its lines switch at the
same time.  This implies that:
- the decoding of TTC_Brcst is confined in the domain of TTC_CK40Des1
- the pulses crossing the domains are:
      TTC_L1A, TTC_BCntRes,  TTC_EvCntRes, TTC_Hard_rst,
      TTC_Soft_rst, TTC_Start, TTC_Stop, TTC_BC0, TTC_StRq;
- BCN, EvN, OrN will be generated in the domain of clk.
- LastTTCcommand is a quasi-static signal, no special care.
Note that TTC_CK40Des1 has a jitter of ~ 1 ns pk-pk,
this could create an L1A pulse shorter than a cycle of clk;
we do a digital edge-detection with the 80MHz clk2x.
*****************************************************************************/

module decodeTTC
(   
 input TTC_CK40Des1, 
 input reset, 
 input [7:2] TTC_Brcst,
 input TTC_BrcstStr,   
 output reg [15:0]
        TTC_test0, TTC_test1, TTC_test2, TTC_test3,
        TTC_test4, TTC_test5, TTC_test6, TTC_test7,
        TTC_test8, TTC_test9, TTC_testA, TTC_testB,
        TTC_testC, TTC_testD, TTC_testE, TTC_testF,
 output reg [7:0] LastTTCcommand = 'h00 
 );

 parameter 
  TTC0                     = 4'h0,
  TTC_BC0_code             = 4'h1,
  TTC_TestEnable_code      = 4'h2,
  TTC_RstOrbitCntr_code    = 4'h3,
  TTC_PrivateGap_code      = 4'h4,
  TTC_Resync_code          = 4'h5,
  TTC_PrivateOrbit_code    = 4'h6,
  TTC_Hard_rst_code        = 4'h7,
  TTC_Soft_rst_code        = 4'h8,   
  TTC_Start_code           = 4'h9,
  TTC_StRq_code            = 4'hA,  
  TTC_Stop_code            = 4'hB,
  TTC_AsyncFifoRst_code    = 4'hC,
  TTC_CalibTrig_code       = 4'hD,
  TTCE                     = 4'hE,
  TTCF                     = 4'hF;  
  
 wire [3:0] TTC_Broadcast_Bits = {TTC_Brcst[7],TTC_Brcst[6],TTC_Brcst[5],TTC_Brcst[3]};
 wire TTC_RstOrbitCntr; // this command is used only in this module
 // internal regs in the domain of TTC_CK40Des1 (t-prefix)
 reg  tRes, tTTC_Hard_rst, tTTC_Soft_rst, tTTC_Resync;
 reg  tTTC_TestEnable, tTTC_RstOrbitCntr, tTTC_CalibTrig;  
 reg  tTTC_Start, tTTC_Stop, tTTC_BC0, tTTC_StRq, tTTC_AsyncFifoRst;

 always @ (posedge TTC_CK40Des1) begin // test Brcst bus
    tRes <= reset; // synchronize reset to TTC_CK40Des1 
    if (tRes) begin 
      TTC_test0 <= 0;  TTC_test1 <= 0;  TTC_test2 <= 0;  TTC_test3 <= 0;
      TTC_test4 <= 0;  TTC_test5 <= 0;  TTC_test6 <= 0;  TTC_test7 <= 0;
      TTC_test8 <= 0;  TTC_test9 <= 0;  TTC_testA <= 0;  TTC_testB <= 0;
      TTC_testC <= 0;  TTC_testD <= 0;  TTC_testE <= 0;  TTC_testF <= 0;
    end
    else if (TTC_BrcstStr)   // Validate brcst bus
         case (TTC_Broadcast_Bits)
            0: TTC_test0 <= TTC_test0 + 1;  
            1: TTC_test1 <= TTC_test1 + 1;
            2: TTC_test2 <= TTC_test2 + 1;
            3: TTC_test3 <= TTC_test3 + 1;
            4: TTC_test4 <= TTC_test4 + 1;
            5: TTC_test5 <= TTC_test5 + 1;
            6: TTC_test6 <= TTC_test6 + 1;
            7: TTC_test7 <= TTC_test7 + 1;
            8: TTC_test8 <= TTC_test8 + 1;
            9: TTC_test9 <= TTC_test9 + 1;
           10: TTC_testA <= TTC_testA + 1;
           11: TTC_testB <= TTC_testB + 1;
           12: TTC_testC <= TTC_testC + 1;
           13: TTC_testD <= TTC_testD + 1;
           14: TTC_testE <= TTC_testE + 1;
           15: TTC_testF <= TTC_testF + 1;
         endcase
 end  // always
   
endmodule  